#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_icon=AFStart-Icon.ico
#AutoIt3Wrapper_outfile=afstart_neu.exe
#AutoIt3Wrapper_UseAnsi=y
#AutoIt3Wrapper_Change2CUI=y
#AutoIt3Wrapper_Res_Comment=AFStart provides a command line interface for Airfoil to select audio sources and streaming devices from command line. AFStart also provides auto detection of running audio sources for streaming.
#AutoIt3Wrapper_Res_Description=AFStart - command line interface for Airfoil
#AutoIt3Wrapper_Res_Fileversion=1.0.0.7
#AutoIt3Wrapper_Res_Fileversion_AutoIncrement=y
#AutoIt3Wrapper_Res_LegalCopyright=(c) 2008 Andreas Brner | http://software.andreas-boerner.de
#AutoIt3Wrapper_Res_Field=ProductName|AFStart
#AutoIt3Wrapper_Res_Field=InternalName|AFStart
#AutoIt3Wrapper_Res_Field=CompanyName|Andreas Brner
#AutoIt3Wrapper_Res_Field=OriginalFilename|AFStart.exe
#AutoIt3Wrapper_Res_Field=ProductVersion|1.0
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****

#include <Array.au3>
#include <File.au3>
#include <Math.au3>
#Include <String.au3>

$dummy=""
; Hilfe anzeigen, Programm beenden
if cmd_search("h",$dummy) or cmd_search("help",$dummy) Then
	help()
	exit(0)
EndIf

; Einstellungen aus der Registrierung lesen
Global $prefs_regpath,$prefs_srcapppath,$prefs_recent[1],$prefs_airfoilpath,$prefs_autotransmit,$prefs_volume[1],$prefs_posx,$prefs_posy
readprefs()

; Info ausgeben, Programm beenden
if cmd_search("info",$dummy) Then
	info()
	exit(0)
EndIf

; /Stop-Option => Airfoil beenden
if cmd_search("stop",$dummy) Then
	exit_airfoil()
	if not @error Then
		Exit(0)
	Else
		Exit(1)
	EndIf
EndIf

; /Restart-Option => Airfoil beenden und mit neuen Optionen neu starten
if cmd_search("restart",$dummy) Then
	exit_airfoil()
	if @error Then
		Exit(1)
	EndIf
EndIf

;;; Pfad zu Airfoil.exe ermitteln
; In der Argumentliste suchen
if cmd_search("AirfoilPath",$dummy) Then
	$airfoilpath=$dummy[$dummy[0]]
else
	$airfoilpath=$prefs_airfoilpath
EndIf
if ((not FileExists($airfoilpath)) or (StringInStr(FileGetAttrib($airfoilpath),"D")>0)) Then
	ConsoleWrite("ER: Airfoil-Path not found: " & $airfoilpath)
	Exit(1)
EndIf

;;; $apppath ermitteln
$apppath=""

;;; Suchliste erstellen
Dim $search[1]
$search[0]=0
; /SrcAppPath-Argumente lesen
$srcapppath=""
cmd_search("SrcAppPath",$srcapppath)
; Falls keine Pfade angegeben wurden, wird "recentlist" als Default gesetzt
if not IsArray($srcapppath) or $srcapppath[0]=0 Then
	Dim $srcapppath[2]
	$srcapppath[0]=1
	$srcapppath[1]="recentlist"
EndIf

; Recent-Liste fr die weitere Verarbeitung kopieren
$reg_recent=$prefs_recent

; SrcAppPath-Argumente im Suchpfad speichern
for $i=1 to $srcapppath[0]
	; wird das Schlsselwort 'recentlist' gefunden, wird die Liste der 
	; zuletzt benutzten Audioquellen in die Suchliste eingefgt
	if StringLower($srcapppath[$i])="recentlist" Then
		_ArrayConcatenate($search,$reg_recent)
		$search[0]=UBound($search)-1
	; ist ein Pfad angegeben, wird dieser an die Suchliste angehngt
	Else
		$search[0]+=1
		ReDim $search[$search[0]+1]
		$search[$search[0]]=$srcapppath[$i]
	endif
Next

; Laufende Prozesse mit der Suchliste abgleichen, ersten gefundenen Prozess als $apppath setzen
for $i=1 to $search[0]
	$j=""
	$path=_PathSplit($search[$i],$j,$j,$j,$j)
	$taskname=$path[3] & $path[4]

	$pids=ProcessList($taskname)
	for $j=1 to $pids[0][0]
		if StringLower(_PidGetPath($pids[$j][1]))=StringLower($search[$i]) Then
			; falls ein laufender Prozess gefunden wurde, $apppath ndern
			$apppath=$search[$i]
			ExitLoop 2
		EndIf
	Next
Next

; Fallback: Wenn ein /SrcAppPath angegeben, dann setzen, 
; ansonsten die zuletzt benutzte Audio Source aus der Registry setzen
if $apppath="" Then
	if $srcapppath[1]<>"recentlist" and $srcapppath[1]<>"" then 
		$apppath=$srcapppath[1]
	Else
		$apppath=$prefs_srcapppath
	EndIf
EndIf

; Wenn alles nichts hilft: Warnung ausgeben, ohne nderung der Recent-Liste weitermachen
if (not FileExists($apppath)) or (StringInStr(FileGetAttrib($apppath),"D")>0) Then
	ConsoleWrite("WR: SrcAppPath-Path not found: " & $apppath)
Else
	; falls $apppath in der RecentX-Liste auftaucht, dort herauslschen ...
	$i=0
	do
		if StringLower($apppath)=StringLower($reg_recent[$i]) Then
			$apppath=$reg_recent[$i]
			_ArrayDelete($reg_recent,$i)
		Else
			$i+=1
		EndIf
	Until $i=UBound($reg_recent)

	; ... und ganz vorne wieder einfgen
	if IsArray($reg_recent) then
		_ArrayInsert($reg_recent,0,$apppath)
	Else
		Dim $reg_recent[1]
		$reg_recent[0]=$apppath
	EndIf

	; RecentX-Eintrge und (neuen) ScrAppPath in die Registrierung zurckschreiben
	for $i=0 to UBound($reg_recent)-1
		RegWrite($prefs_regpath,"Recent" & $i,"REG_SZ",$reg_recent[$i])
	Next
	RegWrite($prefs_regpath,"SrcAppPath","REG_SZ",$apppath)
EndIf   

; SrcType-Wert aus der Registrierung sichern und temporr ndern (falls Airfoil auf die Wiedrgabe eines Input Device konfiguriert ist)
$reg_bak_srctype=RegRead($prefs_regpath,"SrcType")
RegWrite($prefs_regpath,"SrcType","REG_DWORD",1)

;;; An welche Gerte soll bertragen werden?
; Auto-Transmit-Liste aus der Registrierung sichern
$reg_bak_auto=RegRead($prefs_regpath,"Auto")
; In der Argumentliste suchen
if cmd_search("TransmitTo",$dummy) Then
	if $dummy[0]>0 then
		_ArrayDelete($dummy,0)
		$dummy=_ArrayToString($dummy,",")
		$dummy=StringLower(StringReplace($dummy,":","")) & ","
		; Transmit-Liste temporr ndern
		RegWrite($prefs_regpath,"Auto","REG_SZ",$dummy)
	EndIf
EndIf

;;; Lautstrke-Einstellungen fr einzelne Gerte

; Um die Registrierung nicht unntig mit Volume-Schlsseln "vollzumllen", werden
; MAC-Adressen, die vor dem Start von Airfoil nicht in der Registrierung gefunden werden,
; nach Programmstart wieder gelscht. Findet Airfoil das Gert, speichert
; es den aktuellen Volume-Wert am Programmende ohnehin noch einmal selbststndig.
; Wird das Gert mit der angegebenen MAC-Adresse nicht gefunden, bleibt aber kein
; Volume-Wert in der Registrierung zurck.
dim $volume_delete[1]
$volume_delete[0]=0

; In der Argumentliste suchen
if cmd_search("SetVolume",$dummy) Then
	if $dummy[0]>0 then
		for $i=1 to $dummy[0]
			$volume_cmdline=StringSplit($dummy[$i],"=")
			if $volume_cmdline[0]=2 then
				$volume_cmdline[1]=StringUpper(StringReplace($volume_cmdline[1],":","")) ; MAC gro und ohne Doppelpunkte
				$volume_cmdline[2]=_Max(_Min(Number($volume_cmdline[2]),1000),0) ; Volume-Angabe auf Wert zwischen 0..1000 eingrenzen
				; wenn kein bereits vorhandener Volume-Schlssel gefunden wurde, dann den Schlssel zum spteren lschen vormerken
				RegRead($prefs_regpath,$volume_cmdline[1])
				if @error then 
					_ArrayAdd($volume_delete,$volume_cmdline[1])
					$volume_delete[0]+=1
				EndIf
				; Volume-Wert in die Registrierung schreiben
				RegWrite($prefs_regpath,$volume_cmdline[1],"REG_DWORD",$volume_cmdline[2])
			EndIf
		Next
	EndIf
EndIf

;;; Fensterposition
if cmd_search("PosX",$dummy) Then
	if $dummy[0]>0 then
		$pos=_Max(_Min(Number($dummy[$dummy[0]]),@DesktopWidth-10),0) ; Position auf Wert zwischen 0 und etwas weniger als Desktop-Auflsung eingrenzen
		RegWrite($prefs_regpath,"PosX","REG_DWORD",$pos)
	endif
EndIf
if cmd_search("PosY",$dummy) Then
	if $dummy[0]>0 then
		$pos=_Max(_Min(Number($dummy[$dummy[0]]),@DesktopHeight-10),0) ; Position auf Wert zwischen 0 und etwas weniger als Desktop-Auflsung eingrenzen
		RegWrite($prefs_regpath,"PosY","REG_DWORD",$pos)
	endif
EndIf

;;; Tray-Icon anzeigen?
; Wert aus der Registrierung sichern
$reg_bak_showsystray=RegRead($prefs_regpath,"ShowSystray")
if cmd_search("SysTray",$dummy) Then
	if $dummy[0]>0 then
		Switch StringLower($dummy[$dummy[0]])
			Case "on"
				RegWrite($prefs_regpath,"ShowSysTray","REG_DWORD",1)
			Case "off"
				RegWrite($prefs_regpath,"ShowSysTray","REG_DWORD",0)
		EndSwitch
	EndIf
EndIf

; MinimizeSystray-Wert aus der Registrierung sichern (muss temporr gesetzt werden, falls Airfoil minimiert gestartet werden soll)
$reg_bak_minimizesystray=RegRead($prefs_regpath,"MinimizeSystray")

; InjectMode-Wert aus der Registrierung sichern (muss temporr gesetzt werden, um Instant Hijack zu erzwingen)
$reg_bak_injectmode=RegRead($prefs_regpath,"InjectMode")

; Fenster ggf. ins SysTray minimieren
$winmode="show"
if cmd_search("WinMode",$dummy) Then
	if $dummy[0]>0 then
		Switch StringLower($dummy[$dummy[0]])
			Case "minimize"
				$winmode="minimize"
			Case "systray"
				RegWrite($prefs_regpath,"MinimizeSystray","REG_DWORD",1)
				RegWrite($prefs_regpath,"ShowSysTray","REG_DWORD",1)
				$winmode="systray"
		EndSwitch
	EndIf
EndIf


;;; Airfoil starten
Run($airfoilpath)

; Warten, bis das Airfoil-Fenster aufgeht (max. 5 Sekunden)
WinWaitActive("Airfoil","",5)
$win_airfoil=WinGetHandle("Airfoil")

; vorsichtshalber noch ein bisschen lnger warten bis die Einstellungen aus der
; Registrierung gelesen sind und das Fenster vollstndig aufgebaut ist...
Sleep(2000)

; Fenster ggf. minimieren
Switch $winmode
	Case "minimize"
		WinSetState($win_airfoil,"",@SW_MINIMIZE)
	Case "systray"
		WinSetState($win_airfoil,"",@SW_HIDE)
EndSwitch

;;; gesicherte Einstellungen zurckschreiben
; Auto-Transmit-Liste
RegWrite($prefs_regpath,"Auto","REG_SZ",$reg_bak_auto)
; ShowSysTray-Einstellung
RegWrite($prefs_regpath,"ShowSysTray","REG_DWORD",$reg_bak_showsystray)
; MinimizeSystray-Einstellung
RegWrite($prefs_regpath,"MinimizeSystray","REG_DWORD",$reg_bak_minimizesystray)
; InjectMode-Einstellung
RegWrite($prefs_regpath,"InjectMode","REG_DWORD",$reg_bak_injectmode)
; SrcType-Einstellung
RegWrite($prefs_regpath,"SrcType","REG_DWORD",$reg_bak_srctype)
; vor Programmstart unbekannte Volume-Schlssel wieder lschen
for $i=1 to $volume_delete[0]
	RegDelete($prefs_regpath,$volume_delete[$i])
Next

Exit(0)

; ---------------------------------------------------

;;; Kommandozeile auswerten
Func cmd_search($key,ByRef $liste)
	local $i,$arg,$ret=False
	
	Dim $liste[1]
	$liste[0]=0
	$keyword=""
	$key=StringLower($key)
	for $i=1 to $CmdLine[0]
		$arg=$CmdLine[$i]
		if StringLeft($arg,1)="/" Then
			$keyword=StringLower(StringMid($arg,2))
			if $keyword=$key Then $ret=True
		Else
			if $keyword=$key Then
				$liste[0]+=1
				ReDim $liste[$liste[0]+1]
				$liste[$liste[0]]=$arg
			EndIf
		EndIf
	Next
	Return $ret
EndFunc

;;; Airfoil beenden
Func exit_airfoil()
	$errormsg=""
	$win_airfoil=WinGetHandle("Airfoil")
	if not @error Then
		$pid_airfoil=WinGetProcess($win_airfoil)
		$errormsg=""
		; erstmal auf die nette Tour
		WinClose($win_airfoil)
		WinWaitClose($win_airfoil,5)
		if ProcessExists($pid_airfoil) then
			; wenn das nichts hilft => Prozess abschiessen
			$errormsg="WR: Airfoil didn't respond to window close message."
			ProcessClose($pid_airfoil)
			ProcessWaitClose($pid_airfoil,5)
			if ProcessExists($pid_airfoil) then
				$errormsg="ER: AFStart is unable to kill Airfoil."
				SetError(1)
			EndIf
		EndIf
	Else
		$errormsg="WR: Airfoil is not running."
	EndIf
	if $errormsg<>"" then ConsoleWrite($errormsg)
EndFunc

;;; Hilfe ausgeben
Func help()

	$desc =@LF & wrap(FileGetVersion(@ScriptFullPath,"ProductName") & " " & FileGetVersion(@ScriptFullPath,"FileVersion") & " | " & _
			FileGetVersion(@ScriptFullPath,"LegalCopyright")) & @LF & @LF
	$desc&=wrap(FileGetVersion(@ScriptFullPath,"Comments")) & @LF & @LF

	; >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>|
	$desc &= _
	"AFSTART [/SrcAppPath <SrcAppPath>] [/AirfoilPath <AirfoilPath>]" & @LF & _
	"   [/TransmitTo <MAC>] [/SetVolume <MAC>=<volume>] [PosX <x>] [PosY <y>]" & @LF & _
	"   [/SysTray on|off] [/WinMode minimize|systray|show] [/Stop] [/Restart]" & @LF & _
	"   [/h] [/Help] [/Info]" & @LF & @LF & _
	"   /SrcAppPath    Path to the audio source executable." & @LF & _
	"                  If multiple SrcAppPath arguments are given, AFStart" & @LF & _
	"                  uses the first audio source from the list, that it finds" & @LF & _
	"                  running in memory." & @LF & _
	"                  Use the special keyword 'recentlist' to add Airfoil's" & @LF & _
	"                  list of recently used audio sources to the search list." & @LF & _
	"                  If none of the audio sources from the search list is" & @LF & _
	"                  running, Airfoil runs the audio source given as the first" & @LF & _
	"                  /SrcAppPath argument. If no /SrcAppPath argument is" & @LF & _
	"                  used (or 'recentlist' is used alone), Airfoil will run" & @LF & _
	"                  the last used audio source." & @LF & _
	"                  Default: 'recentlist'" & @LF & _
	"   /AirfoilPath   Path to Airfoil.exe" & @LF & _
	"                  Default: current folder and program files folder." & @LF & _
	"   /TransmitTo    MAC address of an AirPort Express device or a computer" & @LF & _ 
	"                  running Airfoil Speakers to transmit to." & @LF & _
	"                  The MAC can be given with or without colons." & @LF & _
	"                  (Example: '00:1e:d3:13:15:64' is equal to'001ed3131564')." & @LF & _
	"                  If used, Airfoil's auto transmit prefs are ignored." & @LF & _
	"                  Multiple TransmitTo arguments can be used." & @LF & _
	"                  Use the special keyword 'computer' to enable local" & @LF & _
	"                  output to 'Computer'." & @LF & _
	"                  Note: The MAC address needed for transmitting to an" & @LF & _
	"                  AirPort Express device is NOT equal to the 'AirPort-ID'" & @LF & _
	"                  shown in tne AirPort configuration tool. You have to" & @LF & _
	"                  decrement the AirPort-ID by one to get the MAC address." & @LF & _
	"   /SetVolume     Set the output volume for an AirPort Express device or a" & @LF & _
	"                  computer running Airfoil Speakers." & @LF & _
	"                  The MAC can be given with or without colons (see above)." & @LF & _
	"                  The volume can take values from 0 to 1000." & @LF & _
	"                  Multiple SetVolume arguments can be used." & @LF & _
	"                  Default: last saved volume" & @LF & _
	"   /PosX, /PosY   Set Airfoil's window position on startup." & @LF & _
	"                  Note that values below zero and above the cuzrrent desktop" & @LF & _
	"                  size will be ignored to prevent the window from being" & @LF & _
	"                  positioned outside the screen." & @LF & _
	"                  Default: last saved position" & @LF & _
	"   /WinMode       Set Airfoil's window state on startup:" & @LF & _
	"                     minimize - Minimize Window to task bar" & @LF & _
	"                     systray  - Minimize Window to system tray" & @LF & _
	"                     show     - Show Window" & @LF & _
	"                  Default: 'show'" & @LF & _
	"   /SysTray       Show (on) / Hide (off) Airfoil icon in system tray." & @LF & _
	"                  Note: '/WinMode systray' overrides '/SysTray off'." & @LF & _
	"                  Default: last saved setting" & @LF & _
	"   /Stop          Exit running Airfoil." & @LF & _
	"   /Restart       Exit running Airfoil and restart with new options." & @LF & _
	"   /Help, /h      Show this help and exit." & @LF & _
	"   /Info          Show current Airfoil settings and exit." & @LF & _
	""
	
	$dummy=DllCall('user32.dll','Int','CharToOem','str',$desc,'str','') ; translate from OEM to ANSI
	ConsoleWrite($dummy[2])

EndFunc

;;; Einstellungen aus der Registrierung lesen
Func readprefs()
	;;; $prefs_regpath
	Global $prefs_regpath="HKEY_CURRENT_USER\Software\Rogue Amoeba\Airfoil\Settings"
	
	$dummy=RegEnumVal("HKEY_CURRENT_USER\Software\Rogue Amoeba\Airfoil\Settings",1)
	if @error Then
		ConsoleWrite("ER: Error reading Airfoil-Prefs. Registry key  not found: " & $prefs_regpath)
		Exit(1)
	EndIf

	;;; REG: SrcAppPath
	Global $prefs_srcapppath=RegRead($prefs_regpath,"SrcAppPath")
	
	;;; REG: RecentX
	$cnt=-1
	Global $prefs_recent[1]
	While True
		$dummy=RegRead($prefs_regpath,"Recent" & $cnt+1)
		if @error<>0 Then ExitLoop

		$cnt+=1
		ReDim $prefs_recent[$cnt+1]
		$prefs_recent[$cnt]=$dummy
	WEnd

	;;; $prefs_airfoilpath
	; im aktuellen Verzeichnis suchen
	Global $prefs_airfoilpath=@ScriptDir & "\Airfoil.exe"
	if not FileExists($prefs_airfoilpath) Then
		; in der Registrierung nach dem Installationspfad suchen
		$dummy=RegRead("HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\Airfoil","UninstallString")
		if StringLeft($dummy,1)='"' Then $dummy=StringTrimRight(StringTrimLeft($dummy,1),1)
		$j=""
		$path=_PathSplit($dummy,$j,$j,$j,$j)
		$prefs_airfoilpath=$path[1] & $path[2] & "Airfoil.exe"
		if not FileExists($prefs_airfoilpath) Then
			$prefs_airfoilpath=@ProgramFilesDir & "\Airfoil\Airfoil.exe"
			if not FileExists($prefs_airfoilpath) Then
				$prefs_airfoilpath=""
			EndIf
		EndIf
	EndIf

	;;; REG: Auto
	; Hinweis: _StringSplit() (UDF-Funktion) liefert im Gegensatz zu StringSplit() (interne Funktion) ein Feld ohne Zhler im nullten Element
	Global $prefs_autotransmit=_StringSplit(RegRead($prefs_regpath,"Auto"),",")
	if StringStripWS($prefs_autotransmit[UBound($prefs_autotransmit)-1],3)="" Then
		_ArrayDelete($prefs_autotransmit,UBound($prefs_autotransmit)-1)
	EndIf

	;;; REG: aabbccddeeff (MAC mit Volume-Wert)
	Global $prefs_volume[1]
	$i=0
	while 1
		$i+=1
		$dummy=RegEnumVal($prefs_regpath,$i)
		if @error then ExitLoop
		
		if StringLen($dummy)=12 and StringIsXDigit($dummy) Then
			_ArrayAdd($prefs_volume,$dummy & " " & RegRead($prefs_regpath,$dummy))
		EndIf		
	WEnd
	_ArrayDelete($prefs_volume,0)

	;;; REG: PosX + PosY
	Global $prefs_posx=RegRead($prefs_regpath,"PosX")
	Global $prefs_posy=RegRead($prefs_regpath,"PosY")
EndFunc

;;; Info zu aktuellen Einstellungen im INI-Format ausgeben
Func info()
	ConsoleWrite("[AirfoilPrefs]" & @CRLF)
	ConsoleWrite("AirfoilPath=" & $prefs_airfoilpath & @CRLF)
	ConsoleWrite("AirfoilPrefs=" & $prefs_regpath & @CRLF)
	ConsoleWrite("SrcAppPath=" & $prefs_srcapppath & @CRLF)
	
	for $i=0 to UBound($prefs_recent)-1
		ConsoleWrite("Recent" & $i & "=" & $prefs_recent[$i] & @CRLF)
	Next
	
	for $i=0 to UBound($prefs_autotransmit)-1
		ConsoleWrite("TransmitTo" & $i+1 & "=" & $prefs_autotransmit[$i] & @CRLF)
	Next
	
	for $i=0 to UBound($prefs_volume)-1
		ConsoleWrite("Volume" & $i+1 & "=" & $prefs_volume[$i] & @CRLF)
	Next
	
	ConsoleWrite("PosX=" & $prefs_posx & @CRLF)
	ConsoleWrite("PosY=" & $prefs_posy & @CRLF)
EndFunc


;;; absoluten Pfad zu einer Process-ID ermitteln
Func _PidGetPath($pid = "", $strComputer = 'localhost')
    If $pid = "" Then $pid = WinGetProcess(WinGetTitle(""))
    $wbemFlagReturnImmediately = 0x10
    $wbemFlagForwardOnly = 0x20
    $colItems = ""
    $objWMIService = ObjGet("winmgmts:\\" & $strComputer & "\root\CIMV2")
    $colItems = $objWMIService.ExecQuery ("SELECT * FROM Win32_Process WHERE ProcessId = " & $pid, "WQL", $wbemFlagReturnImmediately + $wbemFlagForwardOnly)
    If IsObj($colItems) Then
        For $objItem In $colItems
            If $objItem.ExecutablePath Then Return $objItem.ExecutablePath
        Next
    EndIf
EndFunc   ;==>_PidGetPath

; Funktion wrap
; Die Funktion formatiert eine gegebene Zeichenkette auf eine bestimmte Zeilenlnge
; Gegebenenfalls wird die Zeichenkette nach einem Wort umgebrochen
;
; Parameter:
;		$text	-	zu bearbeitender Text
;		$laenge	-	maximale Zeilenlnge
;		$lb     -   Zeilenumbruchzeichen (Voreinstellung: @LF)
;
Func wrap($text, $laenge = 80, $lb=@LF)
	Local $dummy
	$ret = ''
	While StringLen($text) > $laenge
		$dummy = StringLeft($text, $laenge)
		$lastspace = StringInStr($dummy, $lb, 0, -1)
		If $lastspace = 0 Then
			$lastspace = StringInStr($dummy, ' ', 0, -1)
		EndIf
		If $lastspace > 0 Then
			$ret &= StringLeft($dummy, $lastspace - 1) & $lb
			$text = StringMid($text, $lastspace + 1)
		Else
			$lastspace = $laenge - 3
			$ret &= StringLeft($dummy, $lastspace) & '...' & $lb
			$text = StringMid($text, $lastspace)
		EndIf
	WEnd
	If Not StringIsSpace($text) Then $ret &= $text
	Return $ret
EndFunc   ;==>wrap

